/*----------------------------------------------------------------------
  File    : som.h
  Contents: simple self-organizing map management
  Author  : Christian Borgelt
  History : 18.06.2000 file created from file xsom.c
            07.07.2001 extended to 3-dim. reference vectors
            08.07.2001 function som_draw added
            21.12.2001 cube region added
----------------------------------------------------------------------*/
#ifndef __SOM__
#define __SOM__
#if defined SOM_DRAW || defined SOM_MPOST
#include "proj3d.h"
#endif

/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
/* --- neighborhood types --- */
#define SOM_RECT       0        /* rectangular neighborhood function */
#define SOM_GAUSSIAN   1        /* Gaussian    neighborhood function */

/* --- shapes of pattern regions --- */
#define SOM_SQUARE     0        /* square region */
#define SOM_TRIANGLE   1        /* triangular region */
#define SOM_CIRCLE     2        /* circular region */
#define SOM_SPHERE     3        /* surface of a sphere */
#define SOM_PARABOLA   4        /* rotated parabola */
#define SOM_CUBIC      5        /* cubic function */
#define SOM_COSINE     6        /* cosine function */
#define SOM_CUBE       7        /* cosine function */

/*----------------------------------------------------------------------
  Type Definitions
----------------------------------------------------------------------*/
typedef struct {                /* --- reference vector --- */
  double x, y, z;               /* coordinates of reference vector */
  double px, py;                /* coordinates of projection */
  int    proj;                  /* flag for successful projection */
} REFVEC;                       /* (reference vector) */

typedef struct {                /* --- self-organizing map --- */
  int    width, height;         /* width and height (in units) */
  REFVEC *rvecs[1];             /* reference vectors */
} SOM;                          /* (self-organizing map) */

/*----------------------------------------------------------------------
  Self-Organizing Map Functions
----------------------------------------------------------------------*/
extern SOM* som_create (int width, int height);
extern void som_delete (SOM *som);
extern int  som_width  (SOM *som);
extern int  som_height (SOM *som);

extern void som_init   (SOM *som, double randfn(void),
                        double x, double y, double z);
extern void som_train  (SOM *som, double x, double y, double z,
                        double lrate, int nbhood, double radius);

extern int  som_save   (SOM *som, FILE *file);
extern SOM* som_load   (FILE *file);

extern void som_genpat (int shape, double randfn(void),
                        double *x, double *y, double *z);
#ifdef SOM_DRAW
extern void som_draw   (SOM *som, PROJ3D *p3d,
                        void drawfn (double x, double y, int op));
#endif
#ifdef SOM_MPOST
extern void som_mpost  (SOM *som, PROJ3D *p3d, FILE *file);
#endif

/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
#define som_width(s)   ((s)->width)
#define som_height(s)  ((s)->height)

#endif

